// Bi-directional.ino
// ESP32 @ IED 1.8.16

#include <esp_now.h>
#include <WiFi.h>
#include <Wire.h>
#include "uFire_SHT20.h"

#include <Adafruit_GFX.h>
#include <Adafruit_SSD1306.h>

uFire_SHT20 sht20; 

#define SCREEN_WIDTH 128  // OLED display width, in pixels
#define SCREEN_HEIGHT 64  // OLED display height, in pixels

// SSD1306 display @ I2C (SDA->22, SCL->21 pins)
Adafruit_SSD1306 display(SCREEN_WIDTH, SCREEN_HEIGHT, &Wire, -1);

// MAC Address of receiver(s) 
// uint8_t broadcastAddress[]={0x80,0x7D,0x3A,0xDC,0x07,0x1C}; // send to board 22
// uint8_t broadcastAddress[]={0x80,0x7D,0x3A,0xDC,0x67,0x1C}; // send to board 10
uint8_t broadcastAddress[]={0x30,0xAE,0xA4,0xF5,0x7A,0xE8}; // send to board 2 (ESP32 BIG)

// Define variables to store readings 
float temperature, humidity, pressure;
float incomingTemp, incomingHum, incomingPres;

String success; // true if sending was successful

//Structure to send data - must match the receiver structure
typedef struct struct_message 
{ float temp; float hum; float pres;
} struct_message;

// Create a struct_message called SHT20Readings to hold sensor readings
struct_message SHT20Readings;

// Create a struct_message to hold incoming sensor readings
struct_message incomingReadings;

esp_now_peer_info_t peerInfo;

// Callback when data is sent
void OnDataSent(const uint8_t *mac_addr, esp_now_send_status_t status) 
{ Serial.print("\r\nLast Packet Send Status:\t");
  Serial.println(status == ESP_NOW_SEND_SUCCESS ? "Delivery Success" : "Delivery Fail");
  if (status ==0) success = "OK"; else success = "FAIL";
}

// Callback when data is received
void OnDataRecv(const uint8_t * mac, const uint8_t *incomingData, int len) {
  memcpy(&incomingReadings, incomingData, sizeof(incomingReadings));
  Serial.print("Bytes received: "); Serial.println(len);
  incomingTemp=incomingReadings.temp;
  incomingHum=incomingReadings.hum;
}

void setup() 
{ Serial.begin(115200);
  if(!display.begin(SSD1306_SWITCHCAPVCC, 0x3C)) 
  { Serial.println(F("SSD1306 allocation failed")); return; }
  WiFi.mode(WIFI_STA);
  if (esp_now_init() != ESP_OK) {Serial.println("Error initializing ESP-NOW");return;}
  esp_now_register_send_cb(OnDataSent);
  memcpy(peerInfo.peer_addr, broadcastAddress, 6);
  peerInfo.channel = 0; peerInfo.encrypt = false;
  if (esp_now_add_peer(&peerInfo) != ESP_OK){Serial.println("Failed to add peer");return;}
  esp_now_register_recv_cb(OnDataRecv);
  sht20.begin();
}

void loop() 
{ getReadings();
  SHT20Readings.temp=temperature;
  SHT20Readings.hum=humidity;
  SHT20Readings.pres=pressure;

  // Send message via ESP-NOW
  esp_err_t result=esp_now_send(broadcastAddress, (uint8_t *) &SHT20Readings, sizeof(SHT20Readings));
  Serial.println(result==ESP_OK)?("Data Sent"):("Send Fail");
  updateDisplay(); updateSerMon(); delay(1000);
}

void getReadings()
{ sht20.measure_all(); temperature=sht20.tempC; humidity=sht20.RH; pressure=0;
}

void updateDisplay() // Display Readings on OLED Display
{ // Display Readings on OLED Display
  display.clearDisplay(); display.setTextSize(1); display.setTextColor(WHITE);

  display.setCursor(0,0);  display.println("LOCAL"); 
  display.setCursor(0,15); display.print("T="); display.print(temperature,1); display.cp437(true);  display.write(248);  display.print("C");
  display.setCursor(0,25); display.print("H="); display.print(humidity,1);  display.print(" %");
  
  display.setCursor(64,0);  display.println("REMOTE"); 
  display.setCursor(64,15); display.print("T="); display.print(incomingTemp,1); display.cp437(true);  display.write(248);  display.print("C");
  display.setCursor(64,25); display.print("H="); display.print(incomingHum,1);  display.print(" %");
  display.setCursor(64,56); display.print(success);
  
  display.display();
}

void updateSerMon()   // Display Readings in Serial Monitor
{ Serial.println("REMOTE READINGS");
  Serial.print("T: "); Serial.print(incomingReadings.temp,1);  Serial.println(" ºC");
  Serial.print("H: "); Serial.print(incomingReadings.hum,1);  Serial.println(" %");
  // Serial.print("Pressure: ");  Serial.print(incomingReadings.pres);  Serial.println(" hPa");
  Serial.println();

  Serial.println("LOCAL READINGS");
  Serial.print("T: "); Serial.print(temperature,1);  Serial.println(" ºC");
  Serial.print("H: "); Serial.print(humidity,1);  Serial.println(" %");
  // Serial.print("Pressure: ");  Serial.print(pressure);  Serial.println(" hPa");
  Serial.println();
}
